"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.previewApp = void 0;
const cals_1 = require("@cloudbase/cals");
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const chalk_1 = __importDefault(require("chalk"));
const chokidar_1 = __importDefault(require("chokidar"));
const execa_1 = __importDefault(require("execa"));
const fs_extra_1 = __importStar(require("fs-extra"));
const glob_1 = require("glob");
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const common_1 = require("../../common");
const preview_1 = require("../preview");
const SELF_MODE_ROOT_PATH = path_1.default.resolve(os_1.default.homedir(), '.weda-preview');
const SHARE_MODE_ROOT_PATH = path_1.default.resolve(os_1.default.homedir(), '.tcb-app-dev-server');
const DEV_APPS_LIST_JSON = path_1.default.resolve(SHARE_MODE_ROOT_PATH, '.dev-apps-list.json');
const H5_PREVIEW_TMPL_ROOT_PATH = path_1.default.resolve(__dirname, '..', '..', 'template', 'h5_preview');
function getH5AppPath(onlyWatch, nameOrAppId) {
    return onlyWatch
        ? path_1.default.resolve(SHARE_MODE_ROOT_PATH, 'src', 'apps', nameOrAppId, 'h5')
        : path_1.default.resolve(SELF_MODE_ROOT_PATH, nameOrAppId, 'h5');
}
function getMPAppPath(onlyWatch, nameOrAppId) {
    return onlyWatch
        ? path_1.default.resolve(SHARE_MODE_ROOT_PATH, 'src', 'apps', nameOrAppId, 'mp')
        : path_1.default.resolve(SELF_MODE_ROOT_PATH, nameOrAppId, 'mp');
}
function globAsync(filePath) {
    return new Promise((resolve) => {
        (0, glob_1.glob)(filePath, (err, matches) => {
            resolve(matches);
        });
    });
}
function getAppKey(data) {
    return data.name || data.appId || data.id || data.appKey || '';
}
async function buildPreviewH5App(onlyWatch, data) {
    // 复制模板
    const tmplSrc = path_1.default.resolve(H5_PREVIEW_TMPL_ROOT_PATH, onlyWatch ? 'shareDevServer' : 'selfDevServer');
    const destPath = onlyWatch ? SHARE_MODE_ROOT_PATH : data.outputPath;
    await fs_extra_1.default.copy(tmplSrc, destPath);
    if (onlyWatch) {
        // 更新 dev-apps-list.json，记录调试的应用信息
        if (!(0, fs_extra_1.pathExistsSync)(DEV_APPS_LIST_JSON)) {
            (0, fs_extra_1.ensureFileSync)(DEV_APPS_LIST_JSON);
            (0, fs_extra_1.writeJSONSync)(DEV_APPS_LIST_JSON, []);
        }
        const appList = (0, fs_extra_1.readJSONSync)(DEV_APPS_LIST_JSON, 'utf-8');
        if (!appList.includes(getAppKey(data))) {
            appList.push(getAppKey(data));
            (0, fs_extra_1.writeJSONSync)(DEV_APPS_LIST_JSON, appList, 'utf-8');
        }
        await _addApp();
    }
    await partialUpdatePreviewH5AppFiles(tmplSrc, onlyWatch ? getH5AppPath(onlyWatch, getAppKey(data)) : destPath, data);
    async function _addApp() {
        const mainTsxPath = path_1.default.resolve(tmplSrc, 'src', 'main.tsx.tmpl');
        const mainTsxContent = await fs_extra_1.default.readFile(mainTsxPath, 'utf-8');
        const appList = await (0, fs_extra_1.readJSON)(DEV_APPS_LIST_JSON, 'utf-8');
        const replacedMainTsxContent = mainTsxContent
            .replace("// <!-- Don't touch add import here -->", appList
            .map((app) => `const C${app.replace(/-/g, '_')} = React.lazy(() => import('./apps/${app}/h5/src/App'));`)
            .join('\n'))
            .replace("{/* <!-- Don't touch add item here --> */}", appList.map((app) => `<li><Link to="/${app}">${app}</Link></li>`).join('\n'))
            .replace("{/* <!-- Don't touch add route here --> */}", appList.map((app) => `<Route path="/${app}" element={<C${app.replace(/-/g, '_')} />} />`).join('\n'));
        await (0, fs_extra_1.ensureFile)(path_1.default.resolve(SHARE_MODE_ROOT_PATH, 'src', 'main.tsx'));
        await fs_extra_1.default.writeFile(path_1.default.resolve(SHARE_MODE_ROOT_PATH, 'src', 'main.tsx'), replacedMainTsxContent, 'utf-8');
    }
}
async function partialUpdatePreviewH5AppFiles(tmplDir, outputPath, data) {
    // update App.tsx
    const appTsxPath = path_1.default.resolve(tmplDir, 'src', 'App.tsx.tmpl');
    const appTsxContent = await fs_extra_1.default.readFile(appTsxPath, 'utf-8');
    const replacedAppTsxContent = appTsxContent
        .replace('<%= appId %>', data.appId)
        .replace('<%= uin %>', data.uin)
        .replace('<%= envId %>', data.envId);
    await (0, fs_extra_1.ensureFile)(path_1.default.resolve(outputPath, 'src', 'App.tsx'));
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'App.tsx'), replacedAppTsxContent, 'utf-8');
    // update dependencies
    await (0, fs_extra_1.ensureFile)(path_1.default.resolve(outputPath, 'src', 'dependencies.ts'));
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'dependencies.ts'), `
  export default ${JSON.stringify(data.dependencies)}
  `, 'utf-8');
    // update cals
    await (0, fs_extra_1.ensureFile)(path_1.default.resolve(outputPath, 'src', 'cals.ts'));
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'cals.ts'), `
  export default ${JSON.stringify(data.cals)} as any;
  export const sub_cals = ${JSON.stringify(data.sub_cals || [])} as any;
  `, 'utf-8');
}
async function previewApp(ctx, params) {
    var _a;
    const { projectPath } = ctx;
    const { platform = lowcode_builder_1.BuildType.WEB, wxDevtoolPath, onlyWatch = false } = params;
    const data = await _generateBuildData();
    if (platform === lowcode_builder_1.BuildType.WEB) {
        const buildPreviewH5AppData = _getBuildPreviewH5AppData(data);
        const rootPath = getH5AppPath(onlyWatch, getAppKey(data));
        common_1.Logger.info(`👉 构建预览应用中...`);
        await buildPreviewH5App(onlyWatch, buildPreviewH5AppData);
        if (!onlyWatch) {
            // Install dependencies
            common_1.Logger.info(`👉 安装依赖中...`);
            execa_1.default.commandSync('npm i pnpm -g', {
                cwd: rootPath,
                shell: true,
            });
            execa_1.default.commandSync('pnpm install', {
                cwd: rootPath,
                shell: true,
            });
            // start dev server
            common_1.Logger.info(`👉 启动开发服务器`);
            const subprocess = execa_1.default.command('npm run dev', {
                cwd: rootPath,
                shell: true,
            });
            (_a = subprocess.stdout) === null || _a === void 0 ? void 0 : _a.pipe(process.stdout);
        }
    }
    else if (platform === lowcode_builder_1.BuildType.MP) {
        // First build, need to install independencies
        await (0, lowcode_builder_1.buildWedaApp)(data);
        const previewService = new preview_1.PreviewService({
            port: 0,
            mode: 'debug',
            wxDevtoolPath,
        });
        const rootPath = getMPAppPath(onlyWatch, getAppKey(data));
        common_1.Logger.info('👉 开始构建小程序miniprogram_npm包...');
        await previewService.buildMiniprogramNpm(rootPath);
        common_1.Logger.info('👉 开始打开微信开发者工具...');
        await previewService.openIDE(rootPath);
        common_1.Logger.info(`👉 打开微信开发者工具成功。如没有正常打开，可直接手动打开这个目录：${chalk_1.default.bold.yellow(rootPath)}`);
        const { materialsDir } = (0, lowcode_builder_1.getCompileDirs)();
        common_1.Logger.info(`👉 如遇到问题，建议删除组件缓存重试下。删除目录：${chalk_1.default.bold.yellow(materialsDir)}`);
    }
    // Watch code changing in src dir
    chokidar_1.default
        .watch(path_1.default.resolve(projectPath, 'src'), {
        persistent: true,
        ignoreInitial: true,
    })
        .on('all', async (event, filePath) => {
        const data = await _generateBuildData();
        if (platform === lowcode_builder_1.BuildType.WEB) {
            const buildPreviewH5AppData = _getBuildPreviewH5AppData(data);
            const tmplSrc = path_1.default.resolve(H5_PREVIEW_TMPL_ROOT_PATH, onlyWatch ? 'shareDevServer' : 'selfDevServer');
            const destPath = getH5AppPath(onlyWatch, getAppKey(data));
            await partialUpdatePreviewH5AppFiles(tmplSrc, onlyWatch ? getH5AppPath(onlyWatch, getAppKey(data)) : destPath, buildPreviewH5AppData);
        }
        else if (platform === lowcode_builder_1.BuildType.MP) {
            try {
                await (0, lowcode_builder_1.buildWedaApp)(Object.assign({}, data, { ignoreInstall: true }));
            }
            catch (e) {
                console.error(e);
            }
        }
    });
    async function _generateBuildData() {
        var _a, _b, _c, _d;
        const dependencies = await fs_extra_1.default.readJSON(path_1.default.resolve(projectPath, 'dependencies.json'), 'utf8');
        const codeList = await (0, common_1.fileToCode)(projectPath);
        const calsResult = (0, cals_1.codeToCals)(codeList, 'v0');
        const outputPath = platform === lowcode_builder_1.BuildType.MP
            ? getMPAppPath(onlyWatch, getAppKey(calsResult))
            : getH5AppPath(onlyWatch, getAppKey(calsResult));
        const subApps = await globAsync(path_1.default.resolve(projectPath, 'src', 'packages', '*'));
        let subAppCalsList = [];
        if (subApps.length > 0) {
            subAppCalsList = await Promise.all(subApps.map(async (subApp) => {
                const codeList = await (0, common_1.fileToCode)(subApp);
                return (0, cals_1.codeToCals)(codeList, 'v0');
            }));
        }
        const data = {
            cals: calsResult,
            subAppCalsList,
            dependencies,
            appKey: calsResult.id,
            name: calsResult.name || '',
            runtime: lowcode_builder_1.RUNTIME.CLI,
            buildTypeList: [platform],
            mode: lowcode_builder_1.WebpackModeType.NONE,
            domain: ((_a = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _a === void 0 ? void 0 : _a.domain) || '',
            output: { path: outputPath },
            // BuildType.MP 专用
            // @ts-ignore
            resourceAppId: (_b = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _b === void 0 ? void 0 : _b.resourceAppId,
            deployOptions: {
                mode: lowcode_builder_1.DEPLOY_MODE.PREVIEW,
                // @ts-ignore
                mpAppId: (_c = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _c === void 0 ? void 0 : _c.mpAppId,
                // targetMpAppId: 'wx5e6fc5d676b114d7',
            },
            // @ts-ignore
            endpointType: (_d = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _d === void 0 ? void 0 : _d.endpointType,
        };
        return data;
    }
    function _getBuildPreviewH5AppData(data) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        return {
            outputPath: ((_a = data.output) === null || _a === void 0 ? void 0 : _a.path) || '',
            appId: ((_b = data.cals) === null || _b === void 0 ? void 0 : _b.id) || '',
            name: ((_c = data.cals) === null || _c === void 0 ? void 0 : _c.name) || '',
            // @ts-ignore
            envId: ((_e = (_d = data.cals) === null || _d === void 0 ? void 0 : _d.extra) === null || _e === void 0 ? void 0 : _e.envId) || ((_f = data.cals) === null || _f === void 0 ? void 0 : _f.envId) || '',
            uin: ((_h = (_g = data.cals) === null || _g === void 0 ? void 0 : _g.extra) === null || _h === void 0 ? void 0 : _h.uin) || '',
            dependencies: data.dependencies || [],
            cals: data.cals || {},
            sub_cals: data.subAppCalsList || [],
        };
    }
}
exports.previewApp = previewApp;
